@@ -19,32 +19,43 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
 
-from nr.databind.core import Field, Struct
-from nr.interface import implements
-from pydoc_markdown.interfaces import Processor
+import dataclasses
+import typing as t
+
+import docspec
+
+from pydoc_markdown.interfaces import Processor, Resolver
 from pydoc_markdown.contrib.processors.google import GoogleProcessor
 from pydoc_markdown.contrib.processors.pydocmd import PydocmdProcessor
 from pydoc_markdown.contrib.processors.sphinx import SphinxProcessor
 
 
-@implements(Processor)
-class SmartProcessor(Struct):
-  google = Field(GoogleProcessor, default=GoogleProcessor)
-  pydocmd = Field(PydocmdProcessor, default=PydocmdProcessor)
-  sphinx = Field(SphinxProcessor, default=SphinxProcessor)
+@dataclasses.dataclass
+class SmartProcessor(Processor):
+  """
+  This processor picks the #GoogleProcessor, #SphinxProcessor or #PydocmdProcessor after
+  guessing which is appropriate from the syntax it finds in the docstring.
+  """
 
-  def process(self, graph, _resolver):
-    """
-    Preprocesses a given section into its components.
-    """
+  google: GoogleProcessor = dataclasses.field(default_factory=GoogleProcessor)
+  pydocmd: PydocmdProcessor = dataclasses.field(default_factory=PydocmdProcessor)
+  sphinx: SphinxProcessor = dataclasses.field(default_factory=SphinxProcessor)
 
-    graph.visit(self.process_node)
+  def process(self, modules: t.List[docspec.Module], resolver: t.Optional[Resolver]) -> None:
+    docspec.visit(modules, self._process)
 
-  def process_node(self, node):
-    if not node.docstring:
+  def _process(self, obj: docspec.ApiObject):
+    if not obj.docstring:
       return None
-    if self.sphinx.check_docstring_format(node.docstring):
-      return self.sphinx.process_node(node)
-    if self.google.check_docstring_format(node.docstring):
-      return self.google.process_node(node)
-    return self.pydocmd.process_node(node)
+
+    for name in ('google', 'pydocmd', 'sphinx'):
+      indicator = '@doc:fmt:' + name
+      if indicator in obj.docstring:
+        obj.docstring = obj.docstring.replace(indicator, '')
+        return getattr(self, name)._process(obj)
+
+    if self.sphinx.check_docstring_format(obj.docstring):
+      return self.sphinx._process(obj)
+    if self.google.check_docstring_format(obj.docstring):
+      return self.google._process(obj)
+    return self.pydocmd._process(obj)
