@@ -25,11 +25,15 @@ import io
 import sys
 import typing as t
 
+import docspec
 from docspec_python import format_arglist
 
-
 from pydoc_markdown.interfaces import Context, Renderer, Resolver, SourceLinker
-import docspec
+from pydoc_markdown.util.docspec import format_function_signature, is_method
+
+
+def dotted_name(obj: docspec.ApiObject) -> str:
+  return '.'.join(x.name for x in obj.path)
 
 
 @dataclasses.dataclass
@@ -70,6 +74,11 @@ class MarkdownRenderer(Renderer):
   #: the module name. This is enabled by default.
   descriptive_module_title: bool = False
 
+  #: Add the module name as a prefix to class & method names. This module name is
+  #: also rendered as code if #code_headers is enabled. This is enabled
+  #: by default.
+  add_module_prefix: bool = True
+
   #: Add the class name as a prefix to method names. This class name is
   #: also rendered as code if #code_headers is enabled. This is enabled
   #: by default.
@@ -120,11 +129,11 @@ class MarkdownRenderer(Renderer):
   signature_in_header: bool = False
 
   #: Render the vertical bar '|' before function signature. This is enabled by default.
-  signature_with_vertical_bar: bool = True
+  signature_with_vertical_bar: bool = False
 
   #: Include the "def" keyword in the function signature. This is enabled
   #: by default.
-  signature_with_def: bool = False
+  signature_with_def: bool = True
 
   #: Render the class name in the code block for function signature. Note
   #: that this results in invalid Python syntax to be rendered. This is
@@ -134,6 +143,9 @@ class MarkdownRenderer(Renderer):
   #: Render decorators before function definitions.
   signature_with_decorators: bool = True
 
+  #: Render type hints for data elements in the header.
+  render_typehint_in_data_header: bool = False
+
   #: Add the string "python" after the backticks for code blocks. This is
   #: enabled by default.
   code_lang: bool = True
@@ -193,8 +205,7 @@ class MarkdownRenderer(Renderer):
     return self._resolver.reverse_map.get_parent(obj)
 
   def _is_method(self, obj: docspec.ApiObject) -> bool:
-    return isinstance(obj, docspec.Function) and \
-      isinstance(self._get_parent(obj), docspec.Class)
+    return is_method(obj, self._resolver.reverse_map)
 
   def _format_arglist(self, func: docspec.Function) -> str:
     args = func.args[:]
@@ -206,7 +217,10 @@ class MarkdownRenderer(Renderer):
     if level > self.toc_maxdepth:
       return
     object_id = self._resolver.generate_object_id(obj)
-    fp.write('  ' * level + '* [{}](#{})\n'.format(self._escape(obj.name), object_id))
+    title = self._escape(obj.name)
+    if not self.add_module_prefix and isinstance(obj, docspec.Module):
+      title = title.split('.')[-1]
+    fp.write('  ' * level + '* [{}](#{})\n'.format(title, object_id))
     level += 1
     for child in getattr(obj, 'members', []):
       self._render_toc(fp, level, child)
@@ -230,7 +244,7 @@ class MarkdownRenderer(Renderer):
       level = self.header_level_by_type.get(type_name,
         type(self).__dataclass_fields__['header_level_by_type'].default_factory().get(type_name, level))
     if self.insert_header_anchors and not self.html_headers:
-      fp.write('<a name="{}"></a>\n\n'.format(object_id))
+      fp.write('<a id="{}"></a>\n\n'.format(object_id))
     if self.html_headers:
       header_template = '<h{0} id="{1}">{{title}}</h{0}>'.format(level, object_id)
     else:
@@ -247,7 +261,7 @@ class MarkdownRenderer(Renderer):
     if self.signature_with_decorators:
       parts += self._format_decorations(func.decorations)
     if self.signature_python_help_style and not self._is_method(func):
-      parts.append('{} = '.format(func.path()))
+      parts.append('{} = '.format(dotted_name(func)))
     parts += [x + ' ' for x in func.modifiers or []]
     if self.signature_with_def:
       parts.append('def ')
@@ -256,9 +270,7 @@ class MarkdownRenderer(Renderer):
       assert parent, func
       parts.append(parent.name + '.')
     parts.append((override_name or func.name))
-    parts.append('(' + self._format_arglist(func) + ')')
-    if func.return_type:
-      parts.append(' -> {}'.format(func.return_type))
+    parts.append(format_function_signature(func, self._is_method(func)))
     result = ''.join(parts)
     if add_method_bar and self._is_method(func):
       result = '\n'.join(' | ' + l for l in result.split('\n'))
@@ -270,7 +282,7 @@ class MarkdownRenderer(Renderer):
       bases += ', metaclass=' + str(cls.metaclass)
     code = 'class {}({})'.format(cls.name, bases)
     if self.signature_python_help_style:
-      code = cls.path() + ' = ' + code
+      code = dotted_name(cls) + ' = ' + code
     if self.classdef_render_init_signature_if_needed and '__init__' in cls.members:
       code += ':\n '
       if self.signature_with_vertical_bar:
@@ -333,12 +345,21 @@ class MarkdownRenderer(Renderer):
        (self.add_member_class_prefix and isinstance(obj, docspec.Data)):
       title = self._get_parent(obj).name + '.' + title
     elif self.add_full_prefix and not self._is_method(obj):
-      title = obj.path()
-
+      title = dotted_name(obj)
+    if (not self.add_module_prefix and isinstance(obj, docspec.Module)):
+      title = title.split('.')[-1]
     if isinstance(obj, docspec.Function):
       if self.signature_in_header:
         title += '(' + self._format_arglist(obj) + ')'
 
+    if isinstance(obj, docspec.Data) and obj.datatype and self.render_typehint_in_data_header:
+      if self.code_headers:
+        title += f': {obj.datatype}'
+      elif self.html_headers:
+        title += f': <code>{obj.datatype}</code>'
+      else:
+        title += f': `{obj.datatype}`'
+
     if self.code_headers:
       if self.html_headers or self.sub_prefix:
         if self.sub_prefix and '.' in title:
@@ -347,7 +368,7 @@ class MarkdownRenderer(Renderer):
         title = '<code>{}</code>'.format(title)
       else:
         title = '`{}`'.format(title)
-    else:
+    elif not self.html_headers:
       title = self._escape(title)
     if isinstance(obj, docspec.Module) and self.descriptive_module_title:
       title = 'Module ' + title
@@ -385,7 +406,6 @@ class MarkdownRenderer(Renderer):
     return MarkdownReferenceResolver(modules)
 
   def render(self, modules: t.List[docspec.Module]) -> None:
-    self._resolver.update(modules)
     if self.filename is None:
       self.render_to_stream(modules, sys.stdout)
     else:
