@@ -27,21 +27,22 @@ With no arguments it will load the default configuration file. If the
 or a YAML formatted object for the configuration.
 """
 
+import dataclasses
 import logging
 import os
 import sys
 import typing as t
 import webbrowser
+from pathlib import Path
 
 import click
 import yaml
-from databind.core.types import ConcreteType, from_typing
+from databind.core import ConcreteType, dataclass_to_schema
 from docspec import dump_module
 
 from pydoc_markdown import __version__, PydocMarkdown, static
 from pydoc_markdown.contrib.loaders.python import PythonLoader
 from pydoc_markdown.contrib.renderers.markdown import MarkdownRenderer
-from pydoc_markdown.contrib.renderers.mkdocs import MkdocsRenderer
 from pydoc_markdown.interfaces import Context, Server
 from pydoc_markdown.util.watchdog import watch_paths
 
@@ -98,9 +99,8 @@ class RenderSession:
 
     if self.render_toc is not None:
       # Find the #MarkdownRenderer field for this renderer.
-      import pdb; pdb.set_trace()  # TODO
-      for field in from_typing(type(config.renderer)):
-        if field.type == ConcreteType(MarkdownRenderer):
+      for field in dataclass_to_schema(type(config.renderer)).fields.values():
+        if isinstance(field.type, ConcreteType) and field.type.type == MarkdownRenderer:
           markdown: MarkdownRenderer = getattr(config.renderer, field.name)
           break
       else:
@@ -272,6 +272,7 @@ def cli(
         if os.path.isfile(filename):
           error('file already exists: {!r}'.format(filename))
       for filename, content in static.READTHEDOCS_FILES.items():
+        Path(filename).parent.mkdir(exist_ok=True, parents=True)
         with open(filename, 'w') as fp:
           fp.write(content)
         print('created', filename)
